<?php

declare(strict_types=1);

/**
 * @var yii\web\View $this
 * @var \PortalPeople\datadash\models\ConfigForm $model
 */

use yii\helpers\Html;

$this->title = 'Add Database Connection';
?>

<div class="datadash-config-create">
    <h1><?= Html::encode($this->title) ?></h1>

    <?= $this->render('_form', ['model' => $model, 'isNew' => true]) ?>
</div>
